import * as React from "react";
import * as yup from 'yup';
import { DadosPessoais } from "./DadosPessoais";
import { PessoaContacto } from "./PessoaContacto";
import { Cartoes } from "./Cartoes";
import { Box, Container, Grid, Typography, Button, Stepper, Step, StepLabel } from "@mui/material";
import { useFormik } from 'formik';
import { useState } from "react";
import { matchIsValidTel } from 'mui-tel-input';
import axios from "axios";

interface dados {

  tiporegisto: string;
  tipoempresa: string;
  empresa: string;
  nif?: string;
  representante: string;
  telemovelDP: string;
  emailDP: string;
  morada?: string;
  andar?: string;
  pais?: string;
  cidade?: string;
  codigoPostal?: string;


  nomePessoaContacto: string;
  telemovelPC: string;
  emailPessoaContacto: string;

  numcartoes: number;
  nomecartao1: string[];
  notasAdicionais: string;

}

const initialValues: dados = {

  tiporegisto: '',
  tipoempresa: '',
  empresa: '',
  nif: undefined,
  representante: '',
  telemovelDP: '',
  emailDP: '',
  morada: '',
  andar: undefined,
  pais: 'Portugal',
  cidade: '',
  codigoPostal: undefined,

  nomePessoaContacto: '',
  telemovelPC: '',
  emailPessoaContacto: '',

  numcartoes: 1,
  nomecartao1: [],
  notasAdicionais:'',

}

function App() {

  const steps = ["Dados Pessoais", "Pessoa de Contacto", "Cartões"];

  const [validationErrors, setValidationErrors] = useState<string[]>([]);

  const schemaStep1 = yup.object({

    tiporegisto: yup.string().required("*Por favor insira um tipo de registo válido"),
    representante: yup.string().required("*Por favor insira um nome válido"),
    nif: yup.string().matches(/^[0-9]+$/, "*Por favor insira um NIF válido").length(9, "*O NIF deve ser composto por 9 dígitos"),
    telemovelDP: yup.string()
    .required(' ')
    .test('is-valid-phone', "*Por favor insira um número de telemóvel válido", (value) => {
      return matchIsValidTel(value);
    }),
    emailDP: yup.string()
      .email("*Por favor insira um email válido")
      .required("*Por favor insira um email válido"),
    morada: yup.string(),
    andar: yup.string().matches(/^[0-9]+$/, "*Por favor insira um Andar válido"),
    cidade: yup.string().required("*Por favor insira uma cidade válida"),
    pais: yup.string(),
    codigoPostal: yup.string()

  });

  const schemaStep2 = yup.object({

    nomePessoaContacto: yup.string().required("*Por favor insira um nome de empresa válido"),
    telemovelPC: yup.string()
      .required(' ')
      .test('is-valid-phone', '*Por favor insira um número de telemóvel válido', (value) => {
        return matchIsValidTel(value);
      }),
    emailPessoaContacto: yup.string()
      .email("*Por favor insira um email válido")
      .required("*Por favor insira um email válido")
  });

  const schemaStep3 = yup.object({

    numcartoes: yup.number()
      .positive()
      .integer()
      .required("*Por favor insira um numero de cartões válido"),
    nomecartao1: yup.string(),
    nomecartao2: yup.string(),
    nomecartao3: yup.string(),
    nomecartao4: yup.string(),
    nomecartao5: yup.string(),
    notasAdicionais: yup.string()
    
  });

  const validationSchema = yup.object({
    ...schemaStep1.fields,
    ...schemaStep2.fields,
    ...schemaStep3.fields,
  });

  const onSubmit = async (values: dados) => {
    // Extract the specific data you want from the form values
    const extractedData = {
      name: values.empresa,
      address: values.morada,
      taxNumber: values.nif,
      legal: values.representante,
      line: values.andar,
      zipcode: values.codigoPostal,
      location: values.cidade,
      country: values.pais,
      email: values.emailDP,
      contacts: [{
        name: values.nomePessoaContacto,
        email: values.emailPessoaContacto,
        phone: values.telemovelPC.replace(/\s/g, ''),
      }],
      type: values.tipoempresa
    };

    console.log('Dados: ', values);
    
    console.log('Dados API:', extractedData);

    try {
      // Make a POST request using Axios
      const response = await axios.post('https://dev-hgp-sgi.streamline.pt/api/pendings', extractedData);
      
      console.log('Data submitted successfully', response.data);
  
    } catch (error) {
      // Handle errors
      console.error('Error submitting data:', error);
    }

  };

  const formik = useFormik<dados>  ({
    validateOnMount: true,
    validateOnBlur: false,
    initialValues,
    validationSchema,
    onSubmit
  })

  function getStepContent(step: number) {
    switch (step) {
      case 0:
        return <DadosPessoais formik={formik} />;
      case 1:
        return <PessoaContacto formik={formik} />;
      case 2:
        return <Cartoes formik={formik} />;
      default:
        throw new Error("Unknown step");
    }
  }

  const [activeStep, setActiveStep] = React.useState(0);

  const handleNext = async () => {
    console.log(formik.values.tipoempresa);
    try {

      if (formik.values.tiporegisto == '0'){
        await yup.object({
          tipoempresa: yup.string().required("*Por favor insira um tipo de empresa válido"),
        }).validateAt('tipoempresa', formik.values, { abortEarly: false });
      }

      if (formik.values.tipoempresa >= '0'){
        await yup.object({
          empresa: yup.string().required("*Por favor insira nome da empresa válido"),
        }).validateAt('empresa', formik.values, { abortEarly: false });
      }

      if (activeStep === 0) {
        await schemaStep1.validate(formik.values, { abortEarly: false });
      } else if (activeStep === 1) {
        await schemaStep2.validate(formik.values, { abortEarly: false });
      } else if (activeStep === 2) {
        await schemaStep3.validate(formik.values, { abortEarly: false });
      }

      if (formik.values.tiporegisto == '1'){
        formik.values.empresa = '',
        formik.values.nomePessoaContacto = formik.values.representante;
        formik.values.telemovelPC = formik.values.telemovelDP;
        formik.values.emailPessoaContacto = formik.values.emailDP;
        setActiveStep(activeStep + 2);
        setValidationErrors([]);
      }else{
        setActiveStep(activeStep + 1);
        setValidationErrors([]);  
      }
      
    } catch (error) {
      const errors = (error as yup.ValidationError).errors;
      console.error('Validação Falhada:', (error as yup.ValidationError).errors);
      setValidationErrors(errors);
    }
  };

  const handleBack = () => {  
    if (formik.values.tiporegisto == '1'){
      setActiveStep(activeStep - 2);
      setValidationErrors([]);
    }else{
      setActiveStep(activeStep - 1);
      setValidationErrors([]);  
    }
  };

  return (
    <Container maxWidth="xl">
      <Box sx={{ flexGrow: 1 }}>
        <Grid container spacing={2}>
          <Grid item xs={12} md={5}>
            <Typography variant="h4">Registo na Plataforma</Typography>
            <Typography sx={{ mt: 2 }}>
              Oferecemos aos nossos clientes um vasto leque de serviços técnicos
              de engenharia, que vão desde a conceção, instalação, manutenção e
              exploração de instalações e equipamentos de acordo com as
              exigências de cada negócio.
            </Typography>
            <Typography sx={{ mt: 2 }}>
              A High Green Power apresenta soluções personalizadas que vão ao
              encontro de cada cliente, sempre com o intuito de otimizar a
              eficiência com vista à obtenção de uma maior rentabilidade
              financeira, tendo como principal objetivo a satisfação plena dos
              nossos clientes.
            </Typography>
            <Typography variant="h5" sx={{ mt: 2 }}>
              Peça o seu cartão CEME no formulário.
            </Typography>
          </Grid>
          <Grid item xs={12} md={7}>

            <Stepper activeStep={activeStep} sx={{ pt: 3, pb: 5 }}>
              {steps.map((label) => (
                <Step key={label}>
                  <StepLabel>{label}</StepLabel>
                </Step>
              ))}
            </Stepper>
              
            <form onSubmit={formik.handleSubmit}>
  
              <Box sx={{ pb: 2 }}>
                {getStepContent(activeStep)}
              </Box>

              {validationErrors.map((error, index) => (
                <Box 
                  sx={{ mt: 1, color: 'red'}}
                  key={index}>
                  {error}
                </Box>
              ))}

              <Box
                style={{
                  display: 'flex',
                  justifyContent: 'space-between',
                  alignItems: 'center',
                  marginTop: '3rem',
                }}
              >

                {(activeStep > 0) && (activeStep < steps.length) && (
                  <Button
                    type="button"
                    variant="contained"
                    onClick={handleBack}
                  >
                    Retroceder
                  </Button>
                )}
                
                {(activeStep === steps.length - 1) && (
                  <Button
                    type="submit"
                    variant="contained"
                  >
                    Finalizar
                  </Button>
                )}

                {(activeStep < steps.length - 1) && (
                  <Button
                    type="button"
                    variant="contained"
                    onClick={handleNext}
                    style={{ marginLeft: 'auto' }}
                  >
                    Avançar
                  </Button>
                )}

              </Box>

            </form>  

          </Grid>
        </Grid>
      </Box>
    </Container>
  );
}

export default App;
